/*
* Copyright (c) 2007.  Sensoray Co.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Sensoray Company Inc.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * This Filter uses the VLC libraries, which are licensed under GPL and not Copyright
 * by Sensoray.  LIBVLC was designed by the VideoLan team.
 * This file includes libvlc.h, which is GPL and as a result this project inherits the GPL license.
 * The following statement is an opinion only and should not be construed as legal advice:
 *    It is possible that using this filter in commercial applications may be done without requiring your software
 *    to be GPL.  This would only apply if you do not include ANY headers or source from this project as this
 *    project is governed by the GPL.
 *    This software is a separate activeX style component and useful as such.  In order to use in commercial apps,
 *    all calls to this filter must be through activex, etc..  However, the GPL must still be followed and
 *    the GPL license must be shown at installed, mentioning the VLC team and clearly stating the VLC is not 
 *    your copyright.
 *    Part of the above opinion about the GPL license and use of VLC in commercial software has been 
 *    formed from JB Kempf's( a lead developer on VLC) blog.  It should not be taken as 
 *    the final legal opinion on GPL usage as GPL issues are very complex.
 *    The full text of the JBKempf blog in question can be found at the link which follows:
 *    From http://www.jbkempf.com/blog/post/2007/09/19/Misconceptions-about-VLC-1-Building-an-application-on-VLC
 * See file Copying.txt in the VLC distribution this is included under
 * @description:  For using VLC muxing in a DirectShow filter graph.
 * This code is provided as is with no warranties. Extra features(such as VLC streaming) may be 
 * added by the user under the GPL license.
 * The filter uses IFileSinkFilter to set and get the filename.  
 * Disclaimer: This filter was designed for users experienced in DirectShow programming.
 *  (Please do not email Sensoray if you are having trouble using "GraphEdit".)
 *  (GraphEdit was designed for debug purposes only and not for production software)
 * If you want to recording to a new FILE, you MUST first set a new filename using IFileSinkFilter,
 * and then restart the graph with IMediaControl.
 * @caveats: Currently only MPEG1 and MPEG2 are supported with raw 48000 (16bit, stereo) audio.
 * plugins must be installed in %WINDOWS%\dshow_vlc_plugins
 * @author: Sensoray Company Inc. [DA]
 * Begin VLC license
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 */
#include <windows.h>
#include <commdlg.h>
#include <streams.h>
#include <initguid.h>
#include <vlc/libvlc.h>
#include <string.h>
#include <stdio.h>
#include <comdef.h>
#include <d3d9.h>
#include <dvdmedia.h>
#include "srayvlcuids.h"
#include "srayvlcbridge.h"
#include "dmodshow.h"
#include "assert.h"

// our local system definitions
#define VIDSYS_NTSC 2
#define VIDSYS_PAL  1

// our pin types
#define PINTYPE_AUDIO 0
#define PINTYPE_VIDEO 1

// smart pointers
_COM_SMARTPTR_TYPEDEF(IPin, IID_IPin);
_COM_SMARTPTR_TYPEDEF(IEnumMediaTypes, IID_IEnumMediaTypes);


#ifdef _DEBUG
#define OUTPUTDEBUGSTRING( strArg ) OutputDebugString( strArg )

#define OUTPUTDEBUGSTRING1( strArg1, param) { \
        char strDbg[256]; \
        sprintf(strDbg, strArg1, param); \
        OutputDebugString( strDbg) ; \
        } \

#define OUTPUTDEBUGSTRING2( strArg1, param1,param2) { \
        char strDbg[256]; \
        sprintf(strDbg, strArg1, param1, param2); \
        OutputDebugString( strDbg) ; \
        } \


#define OUTPUTDEBUGSTRING3( strArg1, param1,param2,param3) { \
        char strDbg[256]; \
        sprintf(strDbg, strArg1, param1, param2, param3); \
        OutputDebugString( strDbg) ; \
        } \

#else
#define OUTPUTDEBUGSTRING( str) { }
#define OUTPUTDEBUGSTRING1( str,p1) { }
#define OUTPUTDEBUGSTRING2( str,p1,p2) { }
#define OUTPUTDEBUGSTRING3( str,p1,p2,p3) { }
#endif

// Setup data
const AMOVIESETUP_MEDIATYPE sudVLCPinTypesAudio[2] =
{
    {
    &MEDIATYPE_Audio,            // Major type
    &MEDIASUBTYPE_PCM          // Minor type
    },
    {
    &MEDIATYPE_Audio,            // Major type
    &MEDIASUBTYPE_MPEG1AudioPayload          // Minor type
    },
};

const AMOVIESETUP_MEDIATYPE sudVLCPinTypesVideo[1] =
{
    {
    &MEDIATYPE_Video,            // Major type
    &MEDIASUBTYPE_NULL          // Minor type
    },
};


const AMOVIESETUP_PIN sudVLCPins[] =
{
    {
    L"Video",                   // Pin string name
    FALSE,                      // Is it rendered
    FALSE,                      // Is it an output
    FALSE,                      // Allowed none
    FALSE,                      // Likewise many
    &CLSID_NULL,                // Connects to filter
    NULL,                       // Connects to pin
    1,                          // Number of types
    sudVLCPinTypesVideo        // Pin information 
    },
    {
    L"Audio",                   // Pin string name
    FALSE,                      // Is it rendered
    FALSE,                      // Is it an output
    FALSE,                      // Allowed none
    FALSE,                      // Likewise many
    &CLSID_NULL,                // Connects to filter
    NULL,                  // Connects to pin
    3,                          // Number of types
    sudVLCPinTypesAudio                // Pin information 
    },
};

const AMOVIESETUP_FILTER sudVlcDump = 
{
    &CLSID_SRAYVLCBRIDGE,             // Filter CLSID
    L"Sensoray VLC Bridge",
    MERIT_DO_NOT_USE,           // Filter merit
    2,                          // Number pins
    sudVLCPins                    // Pin details
};


//  Object creation stuff
CFactoryTemplate g_Templates[1]= { L"Sensoray VLC Bridge", &CLSID_SRAYVLCBRIDGE, CSrayVlc::CreateInstance, NULL, &sudVlcDump };

// Count of objects listed in g_cTemplates
int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);


////////////////////////////////////////////////////////////////////////
//
// Exported entry points for registration and unregistration 
// (in this case they only call through to default implementations).
//
////////////////////////////////////////////////////////////////////////

STDAPI DllRegisterServer() 
{
    return AMovieDllRegisterServer2(TRUE);
}

STDAPI DllUnregisterServer() 
{
    return AMovieDllRegisterServer2(FALSE);
}

//
// DllEntryPoint
//
extern "C" BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, LPVOID);

BOOL APIENTRY DllMain(HANDLE hModule, 
                      DWORD  dwReason, 
                      LPVOID lpReserved)
{
    return DllEntryPoint((HINSTANCE)(hModule), dwReason, lpReserved);
}

CSrayVlcFilter::CSrayVlcFilter(CSrayVlc *pDump,
                         LPUNKNOWN pUnk,
                         CCritSec *pLock,
                         HRESULT *phr) :
    CBaseFilter(NAME("Sensoray VLC Bridge"), pUnk, pLock, CLSID_SRAYVLCBRIDGE),
    m_pVLC(pDump)
{
}


CBasePin * CSrayVlcFilter::GetPin(int n)
{
    if (n == 0) {
        return m_pVLC->m_pVidPin;
    } else if(n == 1) {
        return m_pVLC->m_pAudPin;
    }
    // don't expose the output pins.  They are internally connected to the VLC module
    return NULL;
}


int CSrayVlcFilter::GetPinCount()
{
    // don't expose the output pins.  They are internally connected to the VLC module
    //OutputDebugString("get pin count\n");
    return 2;
}

void CSrayVlc::ThreadProc() 
{
    // if starting from a separate thread
    StartCapture();
    return; 
}

STDMETHODIMP CSrayVlcFilter::Stop()
{
#define WAIT_CLOSE_TIME 200
#define EXTRA_CLOSE_TIME 30
	HRESULT hr = NOERROR;
    CAutoLock cObjectLock(m_pLock);
    int i=0;
    int numconn = 0;
    // first set all our pins to inactive
    if( m_State != State_Stopped) {

		IFilterGraph *pGraph;
		pGraph = GetFilterGraph();
		assert(pGraph != NULL);
		OutputDebugString("vlc stop\n"); 
		m_pVLC->StopCapture();
		while(i++<WAIT_CLOSE_TIME ) {
			// do not continue until sraybridge VLC module really closed.  
			if( libvlc_get_sraybridge_state( m_pVLC->m_vlc_inst) != 1) {
				// always wait an extra X ms for VLC to clean up.
				// see comments in threads.c (VLC code).
				Sleep(EXTRA_CLOSE_TIME);
				break;
			}
			Sleep(1);
		}
		char tmp[100];
		sprintf(tmp, "[dbg]stopped after___ %d ms\n", i);  
		OutputDebugString(tmp);
		m_pVLC->m_pVidPin->Inactive();
		m_pVLC->m_pAudPin->Inactive();
		m_pVLC->m_pAudOut->Inactive();
		m_pVLC->m_pVidOut->Inactive();


	}
    m_State = State_Stopped;
	OutputDebugString("stopped\n");
    return hr;
}


STDMETHODIMP CSrayVlcFilter::Pause()
{
    HRESULT hr;
    int i=0;
    int numconn = 0;
    int expconn = 0;
#define MAX_WAIT_CONNECT 200
#define EXTRA_CONNECT_TIME 30
    CAutoLock cObjectLock(m_pLock);
    OUTPUTDEBUGSTRING("pause\n");
    expconn += (m_pVLC->m_streammask & STREAMMASK_VIDEO) ? 1 : 0;
    expconn += (m_pVLC->m_streammask & STREAMMASK_AUDIO) ? 1 : 0;
    if( m_pVLC) {
        if(m_State == State_Stopped) {
            OUTPUTDEBUGSTRING("playlist start\n"); 
            // start the VLC capture
            m_pVLC->StartCapture();
            Sleep(0);
            m_pVLC->m_playlist_state = State_Running;
            // wait up to 200ms for VLC threads to connect up the output pin.  
            // VLC module does not hold the IFilterGraph lock... 
            // The consequences of not waiting till the internal outputs are connected
            // a possible 80040209 error and loss of video/audio streaming
            while(i++< MAX_WAIT_CONNECT) {
                numconn = 0;
                // if audio connected wait for the audio connection and the video connection
                if( m_pVLC->m_pVidOut->IsConnected() ) {
                    numconn++;
                }
                if( m_pVLC->m_pAudOut->IsConnected() ) {
                    numconn++;
                }
                if( expconn == numconn ) {
                    char tmp[100];
                    sprintf(tmp, "pins connected after %d ms\n", i);  
                    OutputDebugString(tmp);
					// give VLC an extra 30 ms to start
					Sleep(EXTRA_CONNECT_TIME);
                    break;
                }
                Sleep(1);
            }
			assert( i < MAX_WAIT_CONNECT);
            assert(m_pVLC->m_playlist_item != -1);
			if( m_pVLC->m_playlist_item == -1) {
				OutputDebugString("not started\n");
			}
            if( m_pVLC->m_pVidOut->IsConnected()) {
                m_pVLC->m_pVidOut->Active();
            }
            if(m_pVLC->m_pAudOut->IsConnected()) {
                m_pVLC->m_pAudOut->Active();
            }
        }
    }
    hr = CBaseFilter::Pause();
    return hr;
}


STDMETHODIMP CSrayVlcFilter::Run(REFERENCE_TIME tStart)
{
    CAutoLock cObjectLock(m_pLock);

    return CBaseFilter::Run(tStart);
}


CSrayVlcInputPin::CSrayVlcInputPin(CSrayVlc *pDump,
                  LPUNKNOWN pUnk,
                  CBaseFilter *pFilter,
                  CCritSec *pLock,
                  CCritSec *pReceiveLock,
                  LPCWSTR pPinName,
                  int type,
                  HRESULT *phr) :

    CBaseInputPin(NAME("CSrayVlcInputPin"),
                  pFilter,                   // Filter
                  pLock,                     // Locking
                  phr,                       // Return code
                  pPinName),                 // Pin name
    m_pReceiveLock(pReceiveLock),
    m_pVLC(pDump)
{
    m_type = type;
}


// CheckMediaType
// Check if the pin can support this specific proposed type and format
HRESULT CSrayVlcInputPin::CheckMediaType(const CMediaType *pmt) 
{
    if( m_type == PINTYPE_AUDIO) {
        // Currently only support PCM data input OR MPEG1 layer 2.  
        // Note: preferred input is PCM.  VLC can do a better job of audio encoding than
        // most dshow filters. 
        if( *pmt->Type() == MEDIATYPE_Audio) {
            if( (*pmt->Subtype() != MEDIASUBTYPE_PCM ) && (*pmt->Subtype() != MEDIASUBTYPE_MPEG1AudioPayload ) &&
                (*pmt->Subtype() != MEDIASUBTYPE_MPEG1Packet )) {
                return S_FALSE; 
            }
        } else if( *pmt->Type() == MEDIATYPE_Stream) {
            if( *pmt->Subtype() != MEDIASUBTYPE_MPEG1Audio ) {
                return S_FALSE;
            }
        }
    } else if( m_type == PINTYPE_VIDEO) {
        // MPEG1 and MPEG2 only for now
        if( *pmt->Type() != MEDIATYPE_Video) {
            return S_FALSE;
        }
        // MPEg1 and MPEG2 only
        if( (*pmt->Subtype() != MEDIASUBTYPE_MPEG2_VIDEO) && (*pmt->Subtype() != MEDIASUBTYPE_MPEG1Payload)) {
            return S_FALSE;
        }
    }
    return S_OK;
}


// BreakConnect
// Break a connection
HRESULT CSrayVlcInputPin::BreakConnect()
{
    m_pVLC->DecPins();
    if( m_type == PINTYPE_AUDIO) {
        m_pVLC->m_streammask &= ~ STREAMMASK_AUDIO;
        FreeMediaType( m_pVLC->m_audMT);
    } else if( m_type == PINTYPE_VIDEO) {
        m_pVLC->m_streammask &= ~ STREAMMASK_VIDEO;
        FreeMediaType( m_pVLC->m_vidMT);
    }
    return CBaseInputPin::BreakConnect();
}


// no blocking
STDMETHODIMP CSrayVlcInputPin::ReceiveCanBlock()
{
    return S_FALSE;
}


STDMETHODIMP CSrayVlcInputPin::Receive(IMediaSample *pSample)
{
    HRESULT hr;
    CheckPointer(pSample,E_POINTER);
    CAutoLock lock(m_pReceiveLock);
    hr = CBaseInputPin::Receive( pSample);
    if( FAILED(hr)) {
        OutputDebugString("failed receive\n");
        return hr;
    }
#ifdef _DEBUG
    REFERENCE_TIME tStart, tStop;
    // debug the timestamps if desired
    pSample->GetTime(&tStart, &tStop);
#endif
    if( m_type == PINTYPE_VIDEO) {
            hr = m_pVLC->m_pVidOut->Deliver( pSample);
            if( FAILED(hr)) {
                OutputDebugString("dumpvlc: failed to deliver video sample\n");
            }
    }
    else if (m_type == PINTYPE_AUDIO) {
        hr = m_pVLC->m_pAudOut->Deliver( pSample);
    }
    return hr;
} 


STDMETHODIMP CSrayVlcInputPin::EndOfStream(void)
{
    CAutoLock lock(m_pReceiveLock);
    return CBaseInputPin::EndOfStream();
} 



HRESULT CSrayVlcInputPin::CompleteConnect(IPin *pReceivePin)
{
    HRESULT hr = CBaseInputPin::CompleteConnect( pReceivePin);
    if(FAILED(hr) ) {
        return hr;
    }
    m_pVLC->IncPins();
    // complete the VLC module connection if done
    // save the parameters
    if( m_type == PINTYPE_VIDEO) { 
        // start capture done after connected
        HRESULT hr_type;
        AM_MEDIA_TYPE mt;
        hr_type = ConnectionMediaType( &mt);
        if( hr_type == S_OK) {
            CopyMediaType(&m_pVLC->m_vidMT, &mt);
            FreeMediaType(mt);
            // save the parameters in easy to access form
            OUTPUTDEBUGSTRING1("mt is %x\n", m_pVLC->m_vidMT.majortype);
            OUTPUTDEBUGSTRING1("submt is %x\n", m_pVLC->m_vidMT.subtype);
            if( m_pVLC->m_vidMT.subtype == MEDIASUBTYPE_MPEG2_VIDEO) {
                OUTPUTDEBUGSTRING("MPEG2\n");
                m_pVLC->m_compression = MPEG2;
                if( m_pVLC->m_vidMT.formattype == FORMAT_MPEG2Video) {
                    MPEG2VIDEOINFO *fmt = (MPEG2VIDEOINFO *) m_pVLC->m_vidMT.pbFormat;
                    m_pVLC->m_width = fmt->hdr.bmiHeader.biWidth;
                    m_pVLC->m_height = fmt->hdr.bmiHeader.biHeight;
                    m_pVLC->m_bitrate = fmt->hdr.dwBitRate;
                    OUTPUTDEBUGSTRING1("avgtime %d\n", fmt->hdr.AvgTimePerFrame);
                    m_pVLC->m_framerate = (int) ( (REFERENCE_TIME) 10010000000/ fmt->hdr.AvgTimePerFrame);
                    OUTPUTDEBUGSTRING2("avgtime %d, fr %d\n", fmt->hdr.AvgTimePerFrame, m_pVLC->m_framerate);
                }
            } else if (m_pVLC->m_vidMT.subtype == MEDIASUBTYPE_MPEG1Payload) {
                OUTPUTDEBUGSTRING("MPEG1\n");
                m_pVLC->m_compression = MPEG1;
                if( m_pVLC->m_vidMT.formattype == FORMAT_MPEGVideo) {
                    MPEG1VIDEOINFO* fmt = (MPEG1VIDEOINFO*)m_pVLC->m_vidMT.pbFormat;
                    m_pVLC->m_width = fmt->hdr.bmiHeader.biWidth;
                    m_pVLC->m_height = fmt->hdr.bmiHeader.biHeight;
                    m_pVLC->m_bitrate = fmt->hdr.dwBitRate;
                    m_pVLC->m_framerate = (int) ( (REFERENCE_TIME) 10010000000/ fmt->hdr.AvgTimePerFrame);
                    OUTPUTDEBUGSTRING2("avgtime %d, fr %d\n", fmt->hdr.AvgTimePerFrame, m_pVLC->m_framerate);
                }
            }
            else {
                m_pVLC->m_compression = MPEG4;
                OutputDebugString("MPEG4 VLCMux not debugged or supported yet.  please use AVI mux filter\n");
                if( m_pVLC->m_vidMT.formattype == FORMAT_VideoInfo) {
                    VIDEOINFOHEADER *pvh = (VIDEOINFOHEADER *) m_pVLC->m_vidMT.pbFormat;
                    OUTPUTDEBUGSTRING1("height is %d\n", pvh->bmiHeader.biHeight);
                    OUTPUTDEBUGSTRING1("width is %d\n", pvh->bmiHeader.biWidth);
                    OUTPUTDEBUGSTRING1("bitrate is %d\n", pvh->dwBitRate);
                    m_pVLC->m_height = pvh->bmiHeader.biHeight;
                    m_pVLC->m_width = pvh->bmiHeader.biWidth;
                    m_pVLC->m_bitrate = pvh->dwBitRate;
                    m_pVLC->m_framerate = (int) ( (REFERENCE_TIME) 10010000000/ pvh->AvgTimePerFrame);

                } else if( m_pVLC->m_vidMT.formattype == FORMAT_VideoInfo2) {
                    VIDEOINFOHEADER2 *pvh = (VIDEOINFOHEADER2 *) m_pVLC->m_vidMT.pbFormat;
                    OUTPUTDEBUGSTRING1("height is %d\n", pvh->bmiHeader.biHeight);
                    OUTPUTDEBUGSTRING1("width is %d\n", pvh->bmiHeader.biWidth);
                    OUTPUTDEBUGSTRING1("bitrate is %d\n", pvh->dwBitRate);
                    m_pVLC->m_height = pvh->bmiHeader.biHeight;
                    m_pVLC->m_width = pvh->bmiHeader.biWidth;
                    m_pVLC->m_bitrate = pvh->dwBitRate; 
                    m_pVLC->m_framerate = (int) ( (REFERENCE_TIME) 10010000000/ pvh->AvgTimePerFrame);
                }
            }
        } else {
            m_pVLC->m_vidMT.majortype = MEDIATYPE_Video;
        }
    } else if( m_type == PINTYPE_AUDIO) { 
        // start capture done after connected
        HRESULT hr_type;
        AM_MEDIA_TYPE mt;
        hr_type = ConnectionMediaType( &mt);
        if( hr_type != S_OK) {
            m_pVLC->m_audMT.majortype = MEDIATYPE_Audio;
        } else {
            CopyMediaType(&m_pVLC->m_audMT, &mt);
            FreeMediaType(mt);
            if( m_pVLC->m_audMT.subtype == MEDIASUBTYPE_PCM) {
                if( m_pVLC->m_audMT.formattype == FORMAT_WaveFormatEx) {
                    WAVEFORMATEX *pwave = (WAVEFORMATEX *) m_pVLC->m_audMT.pbFormat;
                    // save the audio parameters.
                    // we encode to the same sample rate, channels, and bits.
                    // This could be extended later if desired to re-sample...
                    m_pVLC->m_aud_enc_samplerate = pwave->nSamplesPerSec;
                    m_pVLC->m_aud_enc_channels = pwave->nChannels;
                    m_pVLC->m_aud_enc_bits = pwave->wBitsPerSample;
                }
            }// save the audio parameters.  gen
        }
    }
    if( hr == 0 ) {
        if( m_type == PINTYPE_AUDIO) {
            m_pVLC->m_streammask |= STREAMMASK_AUDIO;
        } else if( m_type == PINTYPE_VIDEO) {
            m_pVLC->m_streammask |= STREAMMASK_VIDEO;
        }
    }
    return hr;
}

HRESULT CSrayVlcInputPin::Active() 
{
    CAutoLock lock_it(m_pLock);

    HRESULT hr;
    OUTPUTDEBUGSTRING("active\n");
    hr = CBaseInputPin::Active();
    return hr;
}

HRESULT CSrayVlcInputPin::Inactive() 
{
    CAutoLock lock_it(m_pLock);
    OUTPUTDEBUGSTRING("inactive\n");
    return CBaseInputPin::Inactive();
}

//
//  CSrayVlc class
//
CSrayVlc::CSrayVlc(LPUNKNOWN pUnk, HRESULT *phr) :
    CUnknown(NAME("Sensoray VLC Bridge"), pUnk),
    m_pFilter(NULL),
    m_pVidPin(NULL), 
    m_pAudPin(NULL),
    m_pPosition(NULL),
    m_hFile(INVALID_HANDLE_VALUE),
    m_pFileName(0),
    m_fWriteError(0)
{
    ASSERT(phr);
    m_pFilter = new CSrayVlcFilter(this, GetOwner(), &m_Lock, phr);
    if (m_pFilter == NULL) {
        if (phr)
            *phr = E_OUTOFMEMORY;
        return;
    }
    m_num_connected = 0;
    m_pVidPin = new CSrayVlcInputPin(this,GetOwner(),
                               m_pFilter,
                               &m_Lock,
                               &m_ReceiveLockV,
                               L"Video",
                               1,
                               phr);

    if (m_pVidPin == NULL) {
        if (phr)
            *phr = E_OUTOFMEMORY;
        return;
    }
    m_pAudPin = new CSrayVlcInputPin(this,GetOwner(),
                               m_pFilter,
                               &m_Lock,
                               &m_ReceiveLockA,
                               L"Audio",
                               0,
                               phr);

    if (m_pAudPin == NULL) {
        if (phr)
            *phr = E_OUTOFMEMORY;
        return;
    }
 
    m_pVidOut = new CSrayVlcOutputPin( this, m_pFilter, &m_Lock, L"Video", 1, phr);
    m_pAudOut = new CSrayVlcOutputPin( this, m_pFilter, &m_Lock, L"Audio", 0, phr);
    AM_MEDIA_TYPE mt;
    memset(&mt, 0, sizeof( AM_MEDIA_TYPE));
    mt.majortype = MEDIATYPE_Video;
    CMediaType *pmt = new CMediaType(mt);
    m_pVidOut->SetMediaType( pmt);

    memset(&mt, 0, sizeof( AM_MEDIA_TYPE));
    mt.majortype = MEDIATYPE_Audio;
    CMediaType *pamt = new CMediaType(mt);
    m_pAudOut->SetMediaType( pmt);
    char szWindows[MAX_PATH];
    char pluginarg[MAX_PATH+64]; 
    ::GetWindowsDirectory( szWindows, sizeof(szWindows) - 1);
    sprintf( pluginarg, "--plugin-path=%s\\dshow_vlc_plugins\\", szWindows);
    OUTPUTDEBUGSTRING1("plugins %s\n", pluginarg);
    open_vlc_lib(pluginarg);
    m_playlist_state = State_Stopped;
    m_streammask = 0;
    memset( &m_vidMT, 0, sizeof(m_vidMT));
    memset( &m_audMT, 0, sizeof(m_audMT));
    m_aud_enc_bitrate = DEF_AUD_ENC_BITRATE;
    m_aud_enc_samplerate = DEF_AUD_ENC_SAMPLERATE;
    m_aud_enc_bits = DEF_AUD_ENC_BITS;
    m_aud_enc_channels = DEF_AUD_ENC_CHANNELS;

}

// set the filename
STDMETHODIMP CSrayVlc::SetFileName(LPCOLESTR pszFileName,const AM_MEDIA_TYPE *pmt)
{
    FILTER_STATE state = State_Paused;
    CheckPointer(pszFileName,E_POINTER);
    if( m_pFilter) {
        m_pFilter->GetState(1000, &state);
        if( state != State_Stopped) {
            return VFW_E_NOT_STOPPED;
        }
    }
    if(wcslen(pszFileName) > MAX_PATH) {
        return ERROR_FILENAME_EXCED_RANGE;
    }
    // code the filename
    m_pFileName = new WCHAR[1+lstrlenW(pszFileName)];
    if (m_pFileName == 0) {
        return E_OUTOFMEMORY;
    }
    lstrcpyW(m_pFileName,pszFileName);
    return 0;
}


// GetCurFile
// Implemented for IFileSinkFilter support
STDMETHODIMP CSrayVlc::GetCurFile(LPOLESTR * ppszFileName,AM_MEDIA_TYPE *pmt)
{
    CheckPointer(ppszFileName, E_POINTER);
    *ppszFileName = NULL;

    if (m_pFileName != NULL) 
    {
        *ppszFileName = (LPOLESTR)
        QzTaskMemAlloc(sizeof(WCHAR) * (1+lstrlenW(m_pFileName)));

        if (*ppszFileName != NULL) 
        {
            lstrcpyW(*ppszFileName, m_pFileName);
        }
    }

    if(pmt) 
    {
        ZeroMemory(pmt, sizeof(*pmt));
        pmt->majortype = MEDIATYPE_NULL;
        pmt->subtype = MEDIASUBTYPE_NULL;
    }

    return S_OK;

} // GetCurFile

// Destructor

CSrayVlc::~CSrayVlc()
{
	assert( m_playlist_item == -1);
	if( m_playlist_item != -1) {
		OutputDebugString("not stopped\n");
	}
    //StopCapture();
    delete m_pAudPin;
    delete m_pVidPin;
    delete m_pAudOut;
    delete m_pVidOut;
    delete m_pFilter;
    delete m_pPosition;
    delete m_pFileName;
    destroy_vlc_lib();

}



void CSrayVlc::vlc_exception (libvlc_exception_t *excp) { 
    char dbgstr[200];
    if (libvlc_exception_raised (excp)) {
      sprintf(dbgstr, "VLC error: %s\n", libvlc_exception_get_message(excp));
      OutputDebugString(dbgstr);
	  libvlc_exception_clear( excp );
	}
}

void CSrayVlc::open_vlc_lib(char *plugin_path) 
{
#define ARGC 6
    char *init_argv[(ARGC+1)] = { "--intf", "dummy", "--reset-plugins-cache", "--win9x-cv-method=1",
#ifdef SHOW_VLC_DEBUG
                           "--verbose=2",
#else
                           "--verbose=0",
#endif 
                           plugin_path,
                           NULL}; 
    libvlc_exception_init( &m_vlc_excp); 
    m_vlc_inst = libvlc_new( ARGC, init_argv, &m_vlc_excp);
#ifdef SHOW_VLC_DEBUG
    libvlc_showconsole(TRUE);
#endif
    vlc_exception( &m_vlc_excp);
	OutputDebugString("open vlc lib\n");
    OUTPUTDEBUGSTRING1("vlcinst %p\n", m_vlc_inst);
}
 

void CSrayVlc::destroy_vlc_lib()
{
    if( m_vlc_inst) { 
        libvlc_destroy( m_vlc_inst);
        vlc_exception( &m_vlc_excp);
        m_vlc_inst = NULL; 
    }
}

#define SRC_STRING			"dshowbridge:/dev/s22550:streams=%d"
#define SRC_STRING_SIZE		(64)
#define ACCESS_STRING_SIZE  (MAX_PATH+100)
#define DST_STRING_SIZE		(MAX_PATH+400)

static void form_dst_file( char *dst_string, char *aenc, char *mux, int asr, int achan, int abr, char *filename, int b_audio_encode)
{
    char access_string[ACCESS_STRING_SIZE];
    if( b_audio_encode ) {
        // encode the audio
        // 48000 is very common
        sprintf(dst_string, "#transcode{acodec=%s,ab=%d,samplerate=%d,channels=%d}:duplicate{", aenc, abr,asr,achan);
        sprintf(access_string, "dst=std{access=%s,mux=%s,dst=\"%s\"}}", "file",  mux,  filename);
        strcat(dst_string, access_string);
    }
    else {
       sprintf(dst_string, "#duplicate{dst=std{access=%s,mux=%s,dst=\"%s\"}}", "file",  mux,  filename);
    }
    return;
}



// we don't support audio only yet using VLC.  
// We also don't support VES using VLC(no need).  Use standard dump filter for that.
int CSrayVlc::StartCapture( )
{
    char src_string[SRC_STRING_SIZE];
    char dst_string[DST_STRING_SIZE];
    char sout_string[(DST_STRING_SIZE + 64)];
    const char *sout_args[] = { sout_string };
    BOOL bEncodeAudio = FALSE;
    char mux[30];
    char aenc[30]; 
    int bitrate = m_bitrate;
    int encodeType = m_compression;
    TCHAR *pFileName;
    int framerate = m_framerate;

    if( m_streammask & STREAMMASK_AUDIO) {  
        if( m_audMT.subtype == MEDIASUBTYPE_PCM) {
            bEncodeAudio = TRUE;
        }
    } 
    
    // We want to convert an Extended ASCII string to UTF-8 (Multi-byte format)
    // CP_ACP  = ASCII Code page
    // CP_UTF8 = UTF-8 Code page
    // VLC needs our string escaped to UTF8 because the VLC command line is 
    // ascii
    char convert[MAX_PATH];
    if(!WideCharToMultiByte(CP_UTF8,0, m_pFileName,-1,convert,MAX_PATH,0,0)) {
        pFileName = NULL;
    } else {
        pFileName = convert;
    }

    sprintf( src_string, SRC_STRING, m_streammask);
    // audio is PCM.  Needs to be encoded to MPEG1 Layer 2.  (Layer 3 has licencing issues)
    strcpy(aenc, "mpga");
    switch(encodeType) {
        case MPEG1:
            strcpy(mux,"mpeg1");
            break;
        case MPEG2: 
            strcpy(mux,"ps");  
            break; 
#if 0
        case MPEG4:
        case MPEG4DIVX:
            // Note: not debugged
            strcpy(mux,"mp4");
            break;
        case MOTIONJPEG:
            // Note: not debugged
            strcpy(mux, "mjpeg");
            break;
#endif
        default:
            OutputDebugString("mode not supported\n");
            return -1;
    }
    form_dst_file( dst_string, aenc, mux, m_aud_enc_samplerate, m_aud_enc_channels, m_aud_enc_bitrate,pFileName, bEncodeAudio);
    sprintf(sout_string,"sout=%s", dst_string);
    OUTPUTDEBUGSTRING1("starting with dest: %s\n", sout_string);
    IPinPtr pVidPin = m_pVidOut;
    IPinPtr pAudPin = m_pAudOut; 
    PIN_DIRECTION pindir;
    if( m_pVidOut ) {
        pVidPin->QueryDirection( &pindir);
        OUTPUTDEBUGSTRING1( "video pin %s \n", pindir == PINDIR_INPUT ? "input" : "output");

    } else {
        OutputDebugString("no vid pin!\n");
    }
	if( m_vlc_inst == NULL) {
		OutputDebugString("no vlc instance\n");
	}
    assert(m_vlc_inst != NULL);
    IFilterGraph *pGraph = m_pFilter->GetFilterGraph();
    OUTPUTDEBUGSTRING1("graph %p\n",m_pFilter->GetFilterGraph());  
    OUTPUTDEBUGSTRING1("vidPin %p\n",pVidPin);   
    libvlc_register_graph(m_vlc_inst, pGraph, pVidPin, pAudPin);
	vlc_exception( &m_vlc_excp);
	/* always make sure playlist is clear */
	libvlc_playlist_clear( m_vlc_inst, &m_vlc_excp);
	vlc_exception( &m_vlc_excp);
    m_playlist_item = libvlc_playlist_add_extended( m_vlc_inst, src_string, NULL, 1, sout_args, &m_vlc_excp);
    vlc_exception( &m_vlc_excp);
    libvlc_playlist_play(m_vlc_inst, m_playlist_item,0,NULL, &m_vlc_excp);    
    vlc_exception( &m_vlc_excp);
    OUTPUTDEBUGSTRING1("startcapture %d\n", m_playlist_item);
    m_playlist_state = State_Running;
	return 0;
}

int CSrayVlc::StopCapture()
{
    if( m_playlist_item!= -1) {
        libvlc_playlist_stop( m_vlc_inst, &m_vlc_excp); 
        vlc_exception( &m_vlc_excp);
        libvlc_playlist_clear( m_vlc_inst, &m_vlc_excp);
        vlc_exception( &m_vlc_excp);
        m_playlist_item = -1;
    }
    m_playlist_state = State_Stopped;
    return 0;
}

// CreateInstance
// Provide the way for COM to create the filter
CUnknown * WINAPI CSrayVlc::CreateInstance(LPUNKNOWN punk, HRESULT *phr)
{
    ASSERT(phr);
    CSrayVlc *pNewObject = new CSrayVlc(punk, phr);
    if (pNewObject == NULL) {
        if (phr)
            *phr = E_OUTOFMEMORY;
    }
    return pNewObject;
}


STDMETHODIMP CSrayVlc::NonDelegatingQueryInterface(REFIID riid, void ** ppv)
{
    CheckPointer(ppv,E_POINTER);
    CAutoLock lock(&m_Lock);
    if (riid == IID_IFileSinkFilter) {
        return GetInterface((IFileSinkFilter *) this, ppv);
    } else if (riid == IID_IConfigAviMux) {
        return GetInterface((IConfigAviMux *) this, ppv);
    }
    else if( (riid == IID_IBaseFilter) || (riid == IID_IMediaFilter) || (riid == IID_IPersist)) {
        return m_pFilter->NonDelegatingQueryInterface(riid, ppv);
    } 

    return CUnknown::NonDelegatingQueryInterface(riid, ppv);

} 


CSrayVlcOutputPin::CSrayVlcOutputPin(CSrayVlc *pDump,
                               CBaseFilter *pFilter,
                               CCritSec *pLock,
                               LPCWSTR pPinName,
                               INT type,
                               HRESULT *phr) :
    CBaseOutputPin(NAME("CSrayVlcOutputPin"), pFilter, &m_cs, phr, pPinName)  
{
    m_pVLC = pDump;
    m_type = type; 
    // we need an output queue because we internally connect to VLC in Pause. 
    // The input pin may receive samples before the output queue is ready.   
    // All video samples are important, especially the first one which MUST contain
    // a sequence header OR the initial mux rate will be incorrect and playback
    // will be adversely affected.
    m_pOutputQueue = NULL;
}


 


// DecideBufferSize
// This has to be present to override the PURE virtual class base function
HRESULT CSrayVlcOutputPin::DecideBufferSize(IMemAllocator *pMemAllocator,
                                        ALLOCATOR_PROPERTIES * ppropInputRequest)
{
    return NOERROR;
} 

HRESULT CSrayVlcOutputPin::DecideAllocator(IMemInputPin *pPin, IMemAllocator **ppAlloc)
{
    OUTPUTDEBUGSTRING("decide allocator\n");
    return CBaseOutputPin::DecideAllocator( pPin, ppAlloc);
}


STDMETHODIMP CSrayVlcOutputPin::QueryAccept( const AM_MEDIA_TYPE *pmt )
{
    OUTPUTDEBUGSTRING("QA");
    return S_OK;
}

HRESULT CSrayVlcOutputPin::AgreeMediaType(IPin *pReceivePin, const CMediaType *pmt)
{
#ifndef _DEBUG
    return CBaseOutputPin::AgreeMediaType( pReceivePin, pmt);
#else
    // START of copy from amfilter.cpp Copyright Microsoft
    ASSERT(pReceivePin);
    IEnumMediaTypes *pEnumMediaTypes = NULL;
    // if the media type is fully specified then use that
    if ( (pmt != NULL) && (!pmt->IsPartiallySpecified())) {
        // if this media type fails, then we must fail the connection
        // since if pmt is nonnull we are only allowed to connect
        // using a type that matches it.
        return AttemptConnection(pReceivePin, pmt);
    }


    /* Try the other pin's enumerator */
    HRESULT hrFailure = VFW_E_NO_ACCEPTABLE_TYPES;

    for (int i = 0; i < 2; i++) {
        HRESULT hr;
        if (i == (int)m_bTryMyTypesFirst) {
            hr = pReceivePin->EnumMediaTypes(&pEnumMediaTypes);
            OUTPUTDEBUGSTRING1("AMT trying other %x\n", hr);
        } else {
            hr = EnumMediaTypes(&pEnumMediaTypes);
            OUTPUTDEBUGSTRING1("AMT trying ours %x\n", hr);
        } 
        if (SUCCEEDED(hr)) {
            ASSERT(pEnumMediaTypes);
            hr = TryMediaTypes(pReceivePin,pmt,pEnumMediaTypes);
            OUTPUTDEBUGSTRING1("AMT TryMediaTypes: %x\n", hr);

            pEnumMediaTypes->Release();
            if (SUCCEEDED(hr)) {
                return NOERROR;
            } else {
                // try to remember specific error codes if there are any
                if ((hr != E_FAIL) &&
                    (hr != E_INVALIDARG) &&
                    (hr != VFW_E_TYPE_NOT_ACCEPTED)) {
                    hrFailure = hr;
                }
            }
        }
    }
    return hrFailure;
    // END of copy from amfilter.cpp base class Copyright Microsoft 
#endif
}

HRESULT CSrayVlcOutputPin::TryMediaTypes(
                        IPin *pReceivePin,      // connect to this pin
                        const CMediaType *pmt,        // proposed type from Connect
                        IEnumMediaTypes *pEnum)   // try this enumerator
{
#ifndef _DEBUG
    return CBaseOutputPin::TryMediaTypes( pReceivePin, pmt, pEnum);
#else
    // START of copy from amfilter.cpp Copyright Microsoft
    /* Reset the current enumerator position */
    HRESULT hr = pEnum->Reset();
    if (FAILED(hr)) {
        OUTPUTDEBUGSTRING("TMT: reset failed\n");
        return hr;
    }

    CMediaType *pMediaType = NULL;
    ULONG ulMediaCount = 0;

    // attempt to remember a specific error code if there is one
    HRESULT hrFailure = S_OK;
    for (;;) {

        /* Retrieve the next media type NOTE each time round the loop the
           enumerator interface will allocate another AM_MEDIA_TYPE structure
           If we are successful then we copy it into our output object, if
           not then we must delete the memory allocated before returning */
        hr = pEnum->Next(1, (AM_MEDIA_TYPE**)&pMediaType,&ulMediaCount);
        OUTPUTDEBUGSTRING2("enum next %d %x\n",ulMediaCount, hr)

        if (hr != S_OK) {
            if (S_OK == hrFailure) {
                hrFailure = VFW_E_NO_ACCEPTABLE_TYPES;
            } 
            return hrFailure;
        }


        ASSERT(ulMediaCount == 1);
        ASSERT(pMediaType);

        // check that this matches the partial type (if any)

        if ((pmt == NULL) ||
            pMediaType->MatchesPartial(pmt)) {

            hr = AttemptConnection(pReceivePin, pMediaType);

            // attempt to remember a specific error code
            if (FAILED(hr) &&
            SUCCEEDED(hrFailure) &&
            (hr != E_FAIL) &&
            (hr != E_INVALIDARG) &&
            (hr != VFW_E_TYPE_NOT_ACCEPTED)) {
                hrFailure = hr;
            }
        } else {
            OUTPUTDEBUGSTRING1("partial match failed %x\n", pmt);

            hr = VFW_E_NO_ACCEPTABLE_TYPES;
        }

        DeleteMediaType(pMediaType);
        if (S_OK == hr) {
            return hr;
        }
    }
    // START of copy from amfilter.cpp Copyright Microsoft
#endif
}

// CheckMediaType
HRESULT CSrayVlcOutputPin::CheckMediaType(const CMediaType *pmt)
{
    return S_OK;
} 

HRESULT CSrayVlcOutputPin::GetMediaType(int iPosition, CMediaType *pmt)
{ 
    UNREFERENCED_PARAMETER(iPosition);
    CheckPointer(pmt,E_POINTER);
    if( m_type == PINTYPE_VIDEO) {
        pmt->SetType( &m_pVLC->m_vidMT.majortype);
        pmt->SetSubtype( &m_pVLC->m_vidMT.subtype);
        pmt->SetFormatType( &m_pVLC->m_vidMT.formattype);
        pmt->SetFormat( m_pVLC->m_vidMT.pbFormat, m_pVLC->m_vidMT.cbFormat);
    } else if( m_type == PINTYPE_AUDIO) {
        pmt->SetType( &m_pVLC->m_audMT.majortype);
        pmt->SetSubtype( &m_pVLC->m_audMT.subtype);
        pmt->SetFormatType( &m_pVLC->m_audMT.formattype);
        pmt->SetFormat( m_pVLC->m_audMT.pbFormat, m_pVLC->m_audMT.cbFormat);
    }
    return 0;
}

// EnumMediaTypes
STDMETHODIMP CSrayVlcOutputPin::EnumMediaTypes(IEnumMediaTypes **ppEnum)
{
    return CBaseOutputPin::EnumMediaTypes( ppEnum); 
}

STDMETHODIMP CSrayVlcOutputPin::Connect(IPin* pReceivePin,const AM_MEDIA_TYPE* pmt)
{
    CheckPointer(pReceivePin,E_POINTER);
    ValidateReadPtr(pReceivePin,sizeof(IPin));
    CAutoLock cObjectLock(m_pLock);
    DisplayPinInfo(pReceivePin);
    if (m_Connected) {
        OUTPUTDEBUGSTRING("already connected\n");
        return VFW_E_ALREADY_CONNECTED;
    }
    // Differs from baseclass in that 
    // we can connect in the pause state if VLC play has been stopped
    // The output pins are not visible to the directshow user but are internal 

    // Find a mutually agreeable media type -
    // Pass in the template media type. If this is partially specified,
    // each of the enumerated media types will need to be checked against
    // it. If it is non-null and fully specified, we will just try to connect
    // with this.
    const CMediaType * ptype = (CMediaType*)pmt;
    HRESULT hr = AgreeMediaType(pReceivePin, ptype); 
    if (FAILED(hr)) {
        OUTPUTDEBUGSTRING("failed connect no agree media type\n");
        // Since the procedure is already returning an error code, there
        // is nothing else this function can do to report the error.
        EXECUTE_ASSERT( SUCCEEDED( BreakConnect() ) );
        return hr;
    }
    OUTPUTDEBUGSTRING("Connection succeeded");
    // create output queue
    // create an output queue
    if(m_pOutputQueue == NULL)
    {
        OUTPUTDEBUGSTRING("new queue\n");
        m_pOutputQueue = new COutputQueue(m_Connected, &hr, TRUE, FALSE);
        if(m_pOutputQueue == NULL) {
            // should NEVER get here.
            OutputDebugString("vlcdump: serious memory problem!\n");
            return E_OUTOFMEMORY;
        }
        // Make sure that the constructor did not return any error
        if(FAILED(hr)) {
            OutputDebugString("vlcdump: serious memory problem!\n");
            delete m_pOutputQueue;
            m_pOutputQueue = NULL;
            return hr;
        }
    }
    return 0;
}

//
// SetMediaType
//
HRESULT CSrayVlcOutputPin::SetMediaType(const CMediaType *pmt)
{
    CAutoLock lock_it(m_pLock);
    OUTPUTDEBUGSTRING("SMT");
    // Make sure that we have an input connected
    if(!m_pVLC->m_pVidPin->IsConnected()) {
        return VFW_E_NOT_CONNECTED;
    }
    // Make sure that the base class likes it
    HRESULT hr = NOERROR;
    hr = CBaseOutputPin::SetMediaType(pmt); 
    if(FAILED(hr))
        return hr;
    return NOERROR;

} // SetMediaType

// CompleteConnect
HRESULT CSrayVlcOutputPin::CompleteConnect(IPin *pReceivePin)
{
    CAutoLock lock_it(m_pLock);
    HRESULT hr = NOERROR;
    OUTPUTDEBUGSTRING("complete connect\n");
    hr = CBaseOutputPin::CompleteConnect(pReceivePin);
    return hr; 

} // CompleteConnect

// Active
HRESULT CSrayVlcOutputPin::Active()
{
    OUTPUTDEBUGSTRING("active outputpin\n");
    // call base class
    return CBaseOutputPin::Active();
} 


// Inactive 
HRESULT CSrayVlcOutputPin::Inactive()
{
    OUTPUTDEBUGSTRING("output inactive\n");
    CAutoLock lock_it(m_pLock);


    // Delete the output queus associated with the pin.
    if(m_pOutputQueue) {
        delete m_pOutputQueue;
        m_pOutputQueue = NULL;
    }

    CBaseOutputPin::Inactive();
    return NOERROR;
}

// Inactive
HRESULT CSrayVlcOutputPin::BreakConnect()
{
    OUTPUTDEBUGSTRING("break connect\n");
    Inactive();
    return S_OK;

}

STDMETHODIMP CSrayVlcOutputPin::ReceiveConnection( IPin * pConnector, const AM_MEDIA_TYPE *pmt)
{
    return CBaseOutputPin::ReceiveConnection( pConnector, pmt);
}


HRESULT CSrayVlcOutputPin::Deliver(IMediaSample *pMediaSample)
{
    CheckPointer(pMediaSample,E_POINTER);
    // if we have an output queue, otherwise use base class
    if(m_pOutputQueue == NULL) {
        return CBaseOutputPin::Deliver( pMediaSample);
    } 
    pMediaSample->AddRef();
    return m_pOutputQueue->Receive(pMediaSample);
}

HRESULT CSrayVlcOutputPin::DeliverEndOfStream()
{
    if(m_pOutputQueue == NULL) {
        return CBaseOutputPin::DeliverEndOfStream();
    }
    m_pOutputQueue->EOS();
    return NOERROR;

}

HRESULT CSrayVlcOutputPin::DeliverBeginFlush()
{
    // Make sure that we have an output queue
    if(m_pOutputQueue == NULL) {
        return CBaseOutputPin::DeliverBeginFlush();
    }

    m_pOutputQueue->BeginFlush();
    return NOERROR;

}

HRESULT CSrayVlcOutputPin::DeliverEndFlush()
{
    // Make sure that we have an output queue
    if(m_pOutputQueue == NULL) {
        return CBaseOutputPin::DeliverEndFlush();
    }
    m_pOutputQueue->EndFlush();
    return NOERROR;

}

