/*
* Copyright (c) 2007.  Sensoray Co.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Sensoray Company Inc.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * This Filter uses the VLC libraries, which are licensed under GPL and not Copyright
 * by Sensoray.  LIBVLC was designed by the VideoLan team.
 * This file includes libvlc.h, which is GPL and as a result this project inherits the GPL license.
 * The following statement is an opinion only and should not be construed as legal advice:
 *    It is possible that using this filter in commercial applications may be done without requiring your software
 *    to be GPL.  This would only apply if you do not include ANY headers or source from this project as this
 *    project is governed by the GPL.
 *    This software is a separate activeX style component and useful as such.  In order to use in commercial apps,
 *    all calls to this filter must be through activex, etc..  However, the GPL must still be followed and
 *    the GPL license must be shown at installed, mentioning the VLC team and clearly stating the VLC is not 
 *    your copyright.
 *    Part of the above opinion about the GPL license and use of VLC in commercial software has been 
 *    formed from JB Kempf's( a lead developer on VLC) blog.  It should not be taken as 
 *    the final legal opinion on GPL usage as GPL issues are very complex.
 *    The full text of the JBKempf blog in question can be found at the link which follows:
 *    From http://www.jbkempf.com/blog/post/2007/09/19/Misconceptions-about-VLC-1-Building-an-application-on-VLC
 * See file Copying.txt in the VLC distribution this is included under
 * @description:  For using VLC muxing in a DirectShow filter graph.
 * This code is provided as is with no warranties. Extra features(such as VLC streaming) may be 
 * added by the user under the GPL license.
 * The filter uses IFileSinkFilter to set and get the filename.  
 * Disclaimer: This filter was designed for users experienced in DirectShow programming.
 *  (Please do not email Sensoray if you are having trouble using "GraphEdit".)
 *  (GraphEdit was designed for debug purposes only and not for production software)
 * If you want to recording to a new FILE, you MUST first set a new filename using IFileSinkFilter,
 * and then restart the graph with IMediaControl.
 * @caveats: Currently only MPEG1 and MPEG2 are supported with raw 48000 (16bit, stereo) audio.
 * plugins must be installed in %WINDOWS%\dshow_vlc_plugins
 * @author: Sensoray Company Inc. [DA]
 * Begin VLC license
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version. 
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 */

#include "vlc/libvlc.h"
#include "assert.h"
class CSrayVlcOutputPin;
class CSrayVlcInputPin;
class CSrayVlc;
class CSrayVlcFilter;

#define MPEG1	0
#define MPEG2   1
#define MPEG4   4
#define MOTIONJPEG 8
#define MPEG4DIVX 9

#define STREAMMASK_AUDIO 1
#define STREAMMASK_VIDEO 2
// Main filter object

class CSrayVlcFilter : public CBaseFilter
{
    CSrayVlc * const m_pVLC;

public:
    CSrayVlcFilter(CSrayVlc *pDump,
                LPUNKNOWN pUnk,
                CCritSec *pLock,
                HRESULT *phr);
    // Pin enumeration
    CBasePin * GetPin(int n);
    int GetPinCount();
    // Open and close the file as necessary
    STDMETHODIMP Run(REFERENCE_TIME tStart);
    STDMETHODIMP Pause();
    STDMETHODIMP Stop();

};


//  Pin object

class CSrayVlcInputPin : public CBaseInputPin
{
    CSrayVlc    * const m_pVLC;           
    CCritSec * const m_pReceiveLock;   
public:

    CSrayVlcInputPin(CSrayVlc *pDump,
                  LPUNKNOWN pUnk,
                  CBaseFilter *pFilter,
                  CCritSec *pLock,
                  CCritSec *pReceiveLock,
                  LPCWSTR pPinName,
                  int type,
                  HRESULT *phr);
    // Do something with this media sample
    STDMETHODIMP Receive(IMediaSample *pSample);
    STDMETHODIMP EndOfStream(void);
    STDMETHODIMP ReceiveCanBlock();
    HRESULT Active();
    HRESULT Inactive();

    HRESULT CheckMediaType(const CMediaType *);
    HRESULT CompleteConnect(IPin *pReceivePin);
    // Break connection
    HRESULT BreakConnect();
    int m_type;

};


// output pin
class CSrayVlcOutputPin : public CBaseOutputPin
{
    friend class CSrayVlcInputPin;
    friend class CSrayVlc;
    CSrayVlc       *m_pVLC;          
    CCritSec    m_cs;
public:
    // Constructor and destructor
    CSrayVlcOutputPin(CSrayVlc *pDump,
                   CBaseFilter *pFilter,
                   CCritSec *pLock,
                   LPCWSTR pPinName,
                   INT type,
                   HRESULT *phr
                   );

    ~CSrayVlcOutputPin( ) 
    {
        //OutputDebugString("dump VLC output pin\n");
    }
    int m_type;
    // Override to enumerate media types
    STDMETHODIMP EnumMediaTypes(IEnumMediaTypes **ppEnum);
    // Check that we can support an output type
    HRESULT CheckMediaType(const CMediaType *pmt);
    HRESULT SetMediaType(const CMediaType *pmt);
    // Negotiation to use our input pins allocator
    HRESULT DecideAllocator(IMemInputPin *pPin, IMemAllocator **ppAlloc);
    HRESULT DecideBufferSize(IMemAllocator *pMemAllocator,
                             ALLOCATOR_PROPERTIES * ppropInputRequest);

    HRESULT Active();
    HRESULT Inactive();
    STDMETHODIMP ReceiveConnection(
        IPin * pConnector,      // this is the initiating connecting pin
        const AM_MEDIA_TYPE *pmt);  // this is the media type we will exchange
    HRESULT CompleteConnect(IPin *pReceivePin);
    HRESULT BreakConnect();
    HRESULT Deliver(IMediaSample *pMediaSample);

    STDMETHODIMP QueryAccept( const AM_MEDIA_TYPE *pmt );
    HRESULT AgreeMediaType(IPin *pReceivePin, const CMediaType *pmt);
    STDMETHODIMP Connect(IPin* pReceivePin,const AM_MEDIA_TYPE* pmt);
    HRESULT GetMediaType(int iPosition, CMediaType *pMediaType);

    HRESULT TryMediaTypes(
                        IPin *pReceivePin,      // connect to this pin
                        const CMediaType *pmt,        // proposed type from Connect
                        IEnumMediaTypes *pEnum);    // try this enumerator
    HRESULT DeliverEndOfStream();
    HRESULT DeliverEndFlush();
    HRESULT DeliverBeginFlush();


protected:
    COutputQueue *m_pOutputQueue;


};

#define DEF_AUD_ENC_BITRATE 192
#define DEF_AUD_ENC_SAMPLERATE 48000
#define DEF_AUD_ENC_BITS 16
#define DEF_AUD_ENC_CHANNELS 2


//  CSrayVlc object which has filter and pin members
class CSrayVlc : public CUnknown,  public IFileSinkFilter
{
    friend class CSrayVlcFilter;
    friend class CSrayVlcInputPin;
    friend class CSrayVlcOutputPin;
    CSrayVlcFilter   *m_pFilter;       // Methods for filter interfaces
    CSrayVlcInputPin *m_pVidPin;          // A simple rendered input pin
    CSrayVlcInputPin *m_pAudPin;          // A simple rendered input pin
    CSrayVlcOutputPin *m_pVidOut;          // A simple rendered input pin
    CSrayVlcOutputPin *m_pAudOut;          // A simple rendered input pin
    CCritSec m_Lock;                // Main renderer critical section
    CCritSec m_ReceiveLockA;         // Sublock for received video samples
    CCritSec m_ReceiveLockV;         // Sublock for received audio samples
    CCritSec  m_csPins; // cs for pin counter
    CPosPassThru *m_pPosition;      // Renderer position controls
    HANDLE   m_hFile;               // Handle to file for dumping
    LPOLESTR m_pFileName;           // The filename where we dump
    BOOL     m_fWriteError;
public:
    DECLARE_IUNKNOWN
    CSrayVlc(LPUNKNOWN pUnk, HRESULT *phr);
    ~CSrayVlc();
    static CUnknown * WINAPI CreateInstance(LPUNKNOWN punk, HRESULT *phr);
    // IFileSinkFilter interface
    STDMETHODIMP SetFileName(LPCOLESTR pszFileName,const AM_MEDIA_TYPE *pmt);
    STDMETHODIMP GetCurFile(LPOLESTR * ppszFileName,AM_MEDIA_TYPE *pmt);

    // start VLC capture
    int StartCapture( );
    int StopCapture( );

    void open_vlc_lib(char *curdir);
    void destroy_vlc_lib();
    void vlc_exception (libvlc_exception_t *excp);

    static DWORD WINAPI ThreadEntry(IN  LPVOID  param)
    {
        (reinterpret_cast <CSrayVlc *> (param)) -> ThreadProc () ;
        return EXIT_SUCCESS ;
    }
    void ThreadProc() ;
    HANDLE hThread;

    // easy access to compression info( without messing with AM_MEDIA_TYPEs)
    AM_MEDIA_TYPE m_vidMT;
    AM_MEDIA_TYPE m_audMT;
    int m_bitrate;
    int m_height;
    int m_width;
    int m_framerate;
    int m_system;
    int m_compression;
    // stream mask
    int m_streammask;  // 2 -- video only,  3--audio plus video (other values NOT supported)

    int m_aud_enc_bitrate;     // if encoding audio, bitrate to use.  default 192
    int m_aud_enc_samplerate;  // audio encoded sample rate
    int m_aud_enc_bits;
    int m_aud_enc_channels;     
    // state of the VLC playlist
    FILTER_STATE m_playlist_state;
    // 
    int GetPinsConnected() {
        CAutoLock lock(&m_csPins);
        return m_num_connected;
    }
    void IncPins() {
        CAutoLock lock(&m_csPins);
        m_num_connected++;

    }
    void DecPins() {
        CAutoLock lock(&m_csPins);
        m_num_connected--;
    }

protected:
    libvlc_instance_t *m_vlc_inst;
    int m_playlist_item;
    libvlc_exception_t m_vlc_excp;
private:
    // number of connected pins
    int m_num_connected;

    // what interfaces we support
    STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void ** ppv);
};

