/*****************************************************************************
 * common.h : DirectShow access module for vlc
 *****************************************************************************
 * Copyright (C) 2002 the VideoLAN team
 * $Id: common.h 16931 2006-10-03 09:04:50Z damienf $
 *
 * Author: Gildas Bazin <gbazin@videolan.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/
// modified by Sensoray for dshowbridge [DA]
/*****************************************************************************
 * Preamble
 *****************************************************************************/
#include <string>
#include <list>
#include <deque>
using namespace std;

#ifndef _MSC_VER
#   include <wtypes.h>
#   include <unknwn.h>
#   include <ole2.h>
#   include <limits.h>
#   ifdef _WINGDI_
#      undef _WINGDI_
#   endif
#   define _WINGDI_ 1
#   define AM_NOVTABLE
#   define _OBJBASE_H_
#   undef _X86_
#   define _I64_MAX LONG_LONG_MAX
#   define LONGLONG long long
#endif

#include <dshow.h>
#include "filter.h"


#define MPEG1      0
#define MPEG2      1
#define MPEG4      2
#define MOTIONJPEG 3
#define MPEG4DIVX  9
#define VIDSYS_NTSC 2
#define VIDSYS_PAL  1
#define FOURCC_AUDIO_PCM     VLC_FOURCC('a','r','a','w')
#define FOURCC_AUDIO_MPEG1   VLC_FOURCC('m','p','g','a')

/****************************************************************************
 * Access descriptor declaration
 ****************************************************************************/
struct access_sys_t
{
    /* These 2 must be left at the beginning */
    vlc_mutex_t             lock;
    vlc_cond_t              wait;
    bool                    b_pts[2];
    int                     i_streams; //(see STREAM_MASK_VIDEO, STREAM_MASK_AUDIO in dshow.cpp)
    int                     i_video_frame_size;
    int                     open;
    es_out_id_t             *p_es_video;
    int                     i_norm;
    int                     i_compression;
    int                     i_framerate;
    int                     i_avgtimeperframe;
    int                     i_vrate;
    int                     i_height;
    int                     i_width;

    // audio props(input)
    int                     i_aud_sr;
    int                     i_aud_channels;
    int                     i_aud_bits;
    int                     i_aud_fourcc;
    int                     i_aud_blockalign;
    int                     i_aud_bps;

    es_out_id_t             *p_es_audio;
    mtime_t                 m_vid_pts_times_ten;  /* PTS times 10 ( so no round off accumulation error) */
    mtime_t                 m_aud_pts;
    int                     m_aud_samples;
    int                     m_vid_samples;

    IPin                   *m_pinAud;
    IPin                   *m_pinVid;
    IFilterGraph           *p_graph;
    CaptureFilter           *p_capture_filter; 

    int                     i_fourcc;
};
